@@ -241,6 +241,28 @@ def event(*event_list):
     return add_attribute
 
 
+def intent(*intent_list):
+    """Make a callable trigger on a message with any of the given intents.
+
+    *Availability: 5.2.0+*
+
+    NOTE: Due to a bug, messages sent with the intent in the CTCP format
+    (rather than IRCv3 message tags) which do not include a message (for
+    example, a message like ``\\x01VERSION\\x01``) are not correctly parsed by
+    the bot, and as such do not get caught by this decorator. This erroneous
+    behavior is kept in 5.x for compatability, but will be rectified in 6.0.
+
+    Additionally in 5.x, a rule or command must be specified in addition to
+    this, even if it is just ``'.*``.
+    """
+    def add_attribute(function):
+        if not hasattr(function, "intents"):
+            function.intents = []
+        function.intents.extend(intent_list)
+        return function
+    return add_attribute
+
+
 def rate(value):
     """Decorator. Equivalent to func.rate = value.
 
@@ -258,25 +280,114 @@ def rate(value):
     return add_attribute
 
 
-def require_privmsg(function):
+def require_privmsg(message=None):
     """
     Decorator, this allows functions to specify if they should be only
     allowed via private message.
+
+    If it is not, `message` will be said if given.
     """
-    @functools.wraps(function)
-    def _nop(*args, **kwargs):
-        # Assign trigger and bot for easy access later
-        bot, trigger = args[0:2]
-        if trigger.is_privmsg:
-            return function(*args, **kwargs)
-        bot.reply('This command can only be executed via PM \
-                  (Private Message)')
-        return
-    return _nop
+    def actual_decorator(function):
+        @functools.wraps(function)
+        def _nop(*args, **kwargs):
+            # Assign trigger and bot for easy access later
+            bot, trigger = args[0:2]
+            if trigger.is_privmsg:
+                return function(*args, **kwargs)
+            else:
+                if message and not callable(message):
+                    bot.say(message)
+        return _nop
+    # Hack to allow decorator without parens
+    if callable(message):
+        return actual_decorator(message)
+    return actual_decorator
+
+
+def require_chanmsg(message=None):
+    """
+    Decorator, this allows functions to specify if they should be only
+    allowed via channel message.
+
+    If it is not, `message` will be said if given.
+    """
+    def actual_decorator(function):
+        @functools.wraps(function)
+        def _nop(*args, **kwargs):
+            # Assign trigger and bot for easy access later
+            bot, trigger = args[0:2]
+            if trigger.is_privmsg:
+                return function(*args, **kwargs)
+            else:
+                if message and not callable(message):
+                    bot.say(message)
+        return _nop
+    # Hack to allow decorator without parens
+    if callable(message):
+        return actual_decorator(message)
+    return actual_decorator
+
+
+def require_privilege(level, message=None):
+    """Decorator. Require at lesat the given channel privilege level to execute
+    the function.
+
+    `level` can be one of the privilege levels defined in this module. If the
+    user does not have the privilege, `message` will be said if given. If it is
+    a private message, no checking will be done."""
+    def actual_decorator(function):
+        @functools.wraps(function)
+        def guarded(bot, trigger, *args, **kwargs):
+            channel_privs = bot.privileges[trigger.sender]
+            allowed = channel_privs.get(trigger.nick, 0) >= level
+            if not trigger.is_privmsg and not allowed:
+                if message and not callable(message):
+                    bot.say(message)
+            else:
+                return function(bot, trigger, *args, **kwargs)
+        return guarded
+    return actual_decorator
+
+
+def require_admin(message=None):
+    """Decorator. Require the user triggering the message to be a bot admin.
+
+    If they are not, `message` will be said if given."""
+    def actual_decorator(function):
+        @functools.wraps(function)
+        def guarded(bot, trigger, *args, **kwargs):
+            if not trigger.admin:
+                if message and not callable(message):
+                    bot.say(message)
+            else:
+                return function(bot, trigger, *args, **kwargs)
+        return guarded
+    # Hack to allow decorator without parens
+    if callable(message):
+        return actual_decorator(message)
+    return actual_decorator
+
+
+def require_owner(message=None):
+    """Decorator. Require the user triggering the message to be the bot owner.
+
+    If they are not, `message` will be said if given."""
+    def actual_decorator(function):
+        @functools.wraps(function)
+        def guarded(bot, trigger, *args, **kwargs):
+            if not trigger.owner:
+                if message and not callable(message):
+                    bot.say(message)
+            else:
+                return function(bot, trigger, *args, **kwargs)
+        return guarded
+    # Hack to allow decorator without parens
+    if callable(message):
+        return actual_decorator(message)
+    return actual_decorator
 
 
 class example(object):
-
     """Decorator. Add an example.
 
     Add an example attribute into a function and generate a test.
